<?php

namespace YottaPayIntegration\Payments\Api\Authorize;

/**
 * Class for process payment result from Yotta Pay API
 */
class Result implements ResultInterface
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \YottaPayIntegration\Payments\Model\Common
     */
    protected $_common;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \YottaPayIntegration\Payments\Model\Common $common
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \YottaPayIntegration\Payments\Model\Common $common
    ) {
        $this->_logger = $logger;
        $this->_common = $common;
    }

    /**
     * Process authorize result
     *
     * @api
     * @param string $userToken
     * @param string $userId
     *
     * @return string
     */
    public function processAuthorizeResult($userToken, $userId)
    {
        try
        {
            $this->_logger->debug('YottaPay | processAuthorizeResult | Start');

            $this->_logger->debug('YottaPay | processAuthorizeResult | userId: ' . $userId);
            $this->_logger->debug('YottaPay | processAuthorizeResult | userToken: ' . $userToken);

            $this->checkRequestData($userId, $userToken);

            //Create credit memo
            $this->checkForStripeAvailable();

            return 'OK';
        }
        catch (\Exception $e)
        {
            $errorMessage = (
                'Error processAuthorizeResult [v. '
                . \YottaPayIntegration\Payments\Model\Common::$moduleVersion
                . ']: '
                . $e->getMessage()
            );

            $this->_logger->error('YottaPay | processAuthorizeResult | ' . $errorMessage);

            throw new \Magento\Framework\Webapi\Exception(
                __($errorMessage),
                0,
                \Magento\Framework\Webapi\Exception::HTTP_INTERNAL_ERROR
            );
        }
        finally
        {
            $this->_logger->debug('YottaPay | processAuthorizeResult | End');
        }
    }

    /**
     * Check required fields, then return the appropriate order
     *
     * @param string $userId
     * @param string $userToken
     */
    private function checkRequestData($userId, $userToken)
    {
        $sourceUserId = $this->_common->getUserId();

        if (empty($sourceUserId))
        {
            throw new \Exception('Empty user_id config param.');
        }

        if ($userId != $sourceUserId)
        {
            throw new \Exception('Unknown userId.');
        }

        if (empty($userToken))
        {
            throw new \Exception('Empty user_token.');
        }

        //Set user_token
        $this->_common->setUserToken($userToken);

        //Clear merchant_identifier
        $this->_common->setMerchantId('');

        //Clear merchant_gateway_key
        $this->_common->setMerchantKey('');
    }

    /**
     * Check stripe available status and set checkout_description_type option
     */
    private function checkForStripeAvailable()
    {
        try
        {
            $sourceUserId = $this->_common->getUserId();

            if (empty($sourceUserId))
            {
                throw new \Exception('Empty user_id config param.');
            }

            $responseData = $this->_common->performGetRequest(
                \YottaPayIntegration\Payments\Model\Common::$stripeIsEnabledEndpoint . '/' . $sourceUserId,
                'https://apiconnect.yottapay.co.uk'
            );

            if (!array_key_exists('result', $responseData))
            {
                throw new \Exception('processResponseData: Missing required parameters in Yotta Pay API response.');
            }

            //Validate result status
            $responseStatus = $responseData['result'];

            if (!(boolval($responseStatus) && strtolower($responseStatus) != 'false'))
            {
                $this->_common->setCheckoutDescriptionType('option_mobile_banking');
            }
            else
            {
                $this->_common->setCheckoutDescriptionType('option_card_payment');
            }
        }
        catch (\Exception $e)
        {
            $this->_logger->debug('YottaPay | checkForStripeAvailable | ' . $errorMessage);
        }
    }
}
