<?php

namespace YottaPayIntegration\Payments\Api\Cancellation;

/**
 * Class for process payment cancellation from Yotta Pay API
 */
class Verification implements VerificationInterface
{
    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $_scopeConfig;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \Magento\Framework\App\Request\Http
     */
    protected $_request;

    /**
     * @var \YottaPayIntegration\Payments\Model\Common
     */
    protected $_common;

    /**
     * @var \YottaPayIntegration\Payments\Api\Cancellation\Data\VerificationResultInterface
     */
    protected $_verificationResultInterfaceFactory;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\App\Request\Http $request
     * @param \YottaPayIntegration\Payments\Model\Common $common
     * @param \YottaPayIntegration\Payments\Api\Cancellation\Data\VerificationResultInterface $verificationResultInterfaceFactory
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\App\Request\Http $request,
        \YottaPayIntegration\Payments\Model\Common $common,
        \YottaPayIntegration\Payments\Api\Cancellation\Data\VerificationResultInterfaceFactory $verificationResultInterfaceFactory
    ) {
        $this->_logger = $logger;
        $this->_request = $request;
        $this->_common = $common;
        $this->_verificationResultInterfaceFactory = $verificationResultInterfaceFactory;
    }

    /**
     * Process payment cancellation

     * @api
     * @return \YottaPayIntegration\Payments\Api\Cancellation\Data\VerificationResultInterface
     */
    public function processCancellationVerification()
    {
        try
        {
            $this->_logger->debug('YottaPay | processCancellationVerification | Start');

            //Return "true" to interrupt the payment as default result
            $interruptPayment = true;

            //Get parameters from request
            $orderIncrementId = $this->_request->getParam('orderIncrementId');
            $this->_logger->debug('YottaPay | processCancellationVerification | For order: #' . $orderIncrementId);

            if (empty($orderIncrementId))
            {
                throw new \Exception('Invalid request data: orderIncrementId.');
            }

            //Load order
            $order = $this->_common->getOrderByIncrementId($orderIncrementId);

            //Verify order payment method
            if ($order->getPayment()->getMethod() != 'yottapay_gateway')
            {
                throw new \Exception(
                    'Invalid order payment method for payment intent (#'
                    . $order->getIncrementId() . '): "'. $order->getPayment()->getMethod() . '".'
                ); 
            }

            //Check order state and YottaPayLongTermConfirmation setting
            if ($order->getState() != 'pending_payment')
            {
                $longTermConfirmation = $this->_common->getParamLongTermConfirmation();

                if (($order->getState() != 'canceled') || !($longTermConfirmation))
                {
                    throw new \Exception(
                        'Invalid order status for payment intent (#'
                        . $order->getIncrementId() . '): "'. $order->getState() . '".'
                    );
                }
            }

            //Return "false" so as not to interrupt the payment
            $interruptPayment = false;
        }
        catch (\Exception $e)
        {
            $this->_logger->error('YottaPay | processCancellationVerification | ' . $e->getMessage());
        }
        finally
        {
            $verificationResult = $this->_verificationResultInterfaceFactory->create();
            $verificationResult->setResult($interruptPayment);

            $this->_logger->debug('YottaPay | processCancellationVerification | result: ' . ($interruptPayment ? 'true' : 'false'));

            $this->_logger->debug('YottaPay | processCancellationVerification | End');

            return $verificationResult;
        }
    }
}
