<?php

namespace YottaPayIntegration\Payments\Api\Payment;

/**
 * Class for process payment result from Yotta Pay API
 */
class Result implements ResultInterface
{
    /**
     * @var \Magento\Sales\Model\Order\Email\Sender\OrderSender
     */
    protected $_orderSender;

    /**
     * @var \Magento\Sales\Model\Order\Email\Sender\InvoiceSender
     */
    protected $_invoiceSender;

    /**
     * @var \Magento\Sales\Model\Order\Payment\Transaction\Builder
     */
    protected $_transactionBuilder;

    /**
     * @var \Magento\Sales\Model\Service\InvoiceService
     */
    protected $_invoiceService;

     /**
     * @var \Magento\CatalogInventory\Api\StockManagementInterface
     */
    protected $_stockManagement;

    /**
     * @var \Magento\CatalogInventory\Model\Indexer\Stock\Processor
     */
    protected $_stockIndexerProcessor;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;
    
    /**
     * @var \Magento\Framework\DB\Transaction
     */
    protected $_dbTransaction;

    /**
     * @var \YottaPayIntegration\Payments\Model\Common
     */
    protected $_common;
    
    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Sales\Model\Order\Email\Sender\OrderSender $orderSender
     * @param \Magento\Sales\Model\Order\Email\Sender\InvoiceSender $invoiceSender
     * @param \Magento\Sales\Model\Order\Payment\Transaction\Builder $transactionBuilder
     * @param \Magento\Sales\Model\Service\InvoiceService $invoiceService
     * @param \Magento\CatalogInventory\Api\StockManagementInterface $stockManagement
     * @param \Magento\CatalogInventory\Model\Indexer\Stock\Processor $stockIndexerProcessor
     * @param \Magento\Framework\DB\Transaction $dbTransaction
     * @param \YottaPayIntegration\Payments\Model\Common $common
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Magento\Sales\Model\Order\Email\Sender\OrderSender $orderSender,
        \Magento\Sales\Model\Order\Email\Sender\InvoiceSender $invoiceSender,
        \Magento\Sales\Model\Order\Payment\Transaction\Builder $transactionBuilder,
        \Magento\Sales\Model\Service\InvoiceService $invoiceService,
        \Magento\CatalogInventory\Api\StockManagementInterface $stockManagement,
        \Magento\CatalogInventory\Model\Indexer\Stock\Processor $stockIndexerProcessor,
        \Magento\Framework\DB\Transaction $dbTransaction,
        \YottaPayIntegration\Payments\Model\Common $common
    ) {
        $this->_logger = $logger;
        $this->_orderSender = $orderSender;
        $this->_invoiceSender = $invoiceSender;
        $this->_transactionBuilder = $transactionBuilder;
        $this->_invoiceService = $invoiceService;
        $this->_stockManagement = $stockManagement;
        $this->_stockIndexerProcessor = $stockIndexerProcessor;
        $this->_dbTransaction = $dbTransaction;
        $this->_common = $common;
    }

    /**
     * Process payment result
     *
     * @param string $orderId
     * @param string $userId
     * @param string $yottaTransactionId
     * @param bool $success
     */
    public function processPaymentResult($orderId, $userId, $yottaTransactionId, $success)
    {
        try
        {
            $this->_logger->debug('YottaPay | processPaymentResult | Start');

            $this->_logger->debug('YottaPay | processPaymentResult | orderId: ' . $orderId);
            $this->_logger->debug('YottaPay | processPaymentResult | userId: ' . $userId);
            $this->_logger->debug('YottaPay | processPaymentResult | yottaTransactionId: ' . $yottaTransactionId);
            $this->_logger->debug('YottaPay | processPaymentResult | success: ' . $success);

            $order = $this->checkRequestData($orderId, $userId, $success);

            $this->checkResponseCode($success, $yottaTransactionId, $order);

            return 'OK';
        }
        catch (\Exception $e)
        {
            $errorMessage = (
                'Error processPaymentResult [v. '
                . \YottaPayIntegration\Payments\Model\Common::$moduleVersion
                . ']: '
                . $e->getMessage()
            );

            $this->_logger->error('YottaPay | processPaymentResult | ' . $errorMessage);

            throw new \Magento\Framework\Webapi\Exception(
                __($errorMessage),
                0,
                \Magento\Framework\Webapi\Exception::HTTP_INTERNAL_ERROR
            );
        }
        finally
        {
            $this->_logger->debug('YottaPay | processPaymentResult | End');
        }
    }

    /**
     * Check required fields, then return the appropriate order
     *
     * @param string $orderId
     * @param string $userId
     * @param bool $success
     * @return \Magento\Sales\Model\Order
     */
    private function checkRequestData($orderId, $userId, $success)
    {
        $sourceUserId = $this->_common->getUserId();

        if (empty($sourceUserId))
        {
            throw new \Exception('Empty user_id config param');
        }

        if ($userId != $sourceUserId)
        {
            throw new \Exception('Unknown userId');
        }

        $splitedShopTransactionIdentifier = explode('-', $orderId);
        $orderIncrementId = end($splitedShopTransactionIdentifier);
        $order = $this->_common->getOrderByIncrementId($orderIncrementId);

        return $order;
    }

    /**
     * Uncancel order
     *
     * @param \Magento\Sales\Model\Order $order
     * @return void
     */
    private function uncancelOrder($order)
    {
        if ($order->getState() != 'canceled')
        {
            throw new \Exception(
                'Invalid order status for processing (#'
                . $order->getIncrementId()
                . '): "'
                . $order->getState()
                . '".'
            );
        }

        if ($order->isCanceled())
        {
            $productStockQty = [];

            foreach ($order->getAllVisibleItems() as $item)
            {
                $productStockQty[$item->getProductId()] = $item->getQtyCanceled();

                foreach ($item->getChildrenItems() as $child)
                {
                    $productStockQty[$child->getProductId()] = $item->getQtyCanceled();
                    $child->setQtyCanceled(0);
                    $child->setTaxCanceled(0);
                    $child->setDiscountTaxCompensationCanceled(0);
                }

                $item->setQtyCanceled(0);
                $item->setTaxCanceled(0);
                $item->setDiscountTaxCompensationCanceled(0);
            }

            $order->setSubtotalCanceled(0);
            $order->setBaseSubtotalCanceled(0);
            $order->setTaxCanceled(0);
            $order->setBaseTaxCanceled(0);
            $order->setShippingCanceled(0);
            $order->setBaseShippingCanceled(0);
            $order->setDiscountCanceled(0);
            $order->setBaseDiscountCanceled(0);
            $order->setTotalCanceled(0);
            $order->setBaseTotalCanceled(0);
            $order->setState("processing")->setStatus("processing");
            $order->addStatusHistoryComment(\YottaPayIntegration\Payments\Model\Common::$historyCommentOrderUncanceled)
                ->setIsCustomerNotified(true)
                ->save();

            /* Reverting inventory */
            $itemsForReindex = $this->_stockManagement->registerProductsSale(
                $productStockQty,
                $order->getStore()->getWebsiteId()
            );

            $productIds = [];

            foreach ($itemsForReindex as $item)
            {
                $item->save();
                $productIds[] = $item->getProductId();
            }

            if (!empty($productIds))
            {
                $this->_stockIndexerProcessor->reindexList($productIds);
            }

            $order->setInventoryProcessed(true);

            $order->save();
        }
        else
        {
            throw new \Exception('We cannot un-cancel this order.');
        }
    }

    /**
     * Send new order email
     *
     * @param \Magento\Sales\Model\Order $order
     * @return void
     */
    private function sendPaymentSuccessEmail($order)
    {
        if (!($order->getEmailSent()))
        {
            $order->setCanSendNewEmailFlag(true);
        }

        if ($order->getCanSendNewEmailFlag())
        {
            try
            {
                return $this->_orderSender->send($order);
            }
            catch (\Exception $e)
            {
                $this->_logger->error('YottaPay | sendPaymentSuccessEmail | ' . $errorMessage);

                return false;
            }
        }
    }

    /**
     * Send new invoice email
     *
     * @param \Magento\Sales\Model\Order\Invoice $invoice
     * @return void
     */
    private function sendInvoiceSuccessEmail($invoice)
    {
        if (!($invoice->getEmailSent()))
        {
            try
            {
                return $this->_invoiceSender->send($invoice);
            }
            catch (\Exception $e)
            {
                $this->_logger->error('YottaPay | sendInvoiceSuccessEmail | ' . $errorMessage);

                return false;
            }
        }
    }

    /**
     * Create transaction and add to payment
     *
     * @param \Magento\Sales\Model\Order $order
     * @param string $yottaTransactionId
     * @return void
     */
    private function addTransactionToPayment($order, $yottaTransactionId)
    {
        // Prepare payment object
        $paymentData = array();
        $payment = $order->getPayment();
        $payment->setLastTransId($yottaTransactionId);
        $payment->setTransactionId($yottaTransactionId);
        $payment->setAdditionalInformation([\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $paymentData]);
        $payment->setParentTransactionId(null);

        // Prepare transaction
        $transaction = $this->_transactionBuilder->setPayment($payment)
            ->setOrder($order)
            ->setTransactionId($yottaTransactionId)
            ->setAdditionalInformation([\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $paymentData])
            ->setFailSafe(true)
            ->build(\Magento\Sales\Model\Order\Payment\Transaction::TYPE_CAPTURE);

        // Save payment and transaction
        $payment->save();
        $transaction->save();
    }

    /**
     * Change order state, send email and add history comment 
     *
     * @param \Magento\Sales\Model\Order $order
     * @param string $yottaTransactionId
     * @return void
     */
    private function processingOrder($order, $yottaTransactionId)
    {
        $order->setState("processing")->setStatus("processing");

        $this->sendPaymentSuccessEmail($order);

        $order->addStatusHistoryComment(
            \YottaPayIntegration\Payments\Model\Common::$historyCommentTransactionCompleted
            . $yottaTransactionId
        )
            ->setIsCustomerNotified(true)
            ->save();

        $order->save();
    }

    /**
     * Invoice order 
     *
     * @param \Magento\Sales\Model\Order $order
     * @return void
     */
    private function invoiceOrder($order)
    {
        $invoice = $this->_invoiceService->prepareInvoice($order);
        $invoice->register();
        $invoice->pay();
        $invoice->save();

        $transactionSave = $this->_dbTransaction->addObject($invoice)->addObject($invoice->getOrder());
        $transactionSave->save();

        $temp_total_amount = $order->getBaseCurrency()->formatTxt($order->getGrandTotal());

        $isNotified = $this->sendInvoiceSuccessEmail($invoice);

        $order->addStatusHistoryComment(
            \YottaPayIntegration\Payments\Model\Common::$historyCommentCaptured
            . $temp_total_amount
        )
            ->setIsCustomerNotified($isNotified)
            ->save();
            
        $order->save();
    }

    /**
     * Check operation result code from request and update order status
     *
     * @param bool $success
     * @param string $yottaTransactionId
     * @param \Magento\Sales\Model\Order $order
     * @return void
     */
    private function checkResponseCode($success, $yottaTransactionId, $order)
    {
        if ($success)
        {
            //Uncancel order if long_term_confirmation param is enabled
            if ($order->getState() != 'pending_payment')
            {
                $long_term_confirmation = $this->_common->getParamLongTermConfirmation();

                if ($long_term_confirmation)
                {
                    $this->uncancelOrder($order);
                }
                else
                {
                    throw new \Exception(
                        'Invalid order status for processing (#'
                        . $order->getIncrementId()
                        . '): "'
                        . $order->getState()
                        . '".'
                    );
                }
            }

            // Prepare payment object
            $this->addTransactionToPayment($order, $yottaTransactionId);

            //Change order state and status
            $this->processingOrder($order, $yottaTransactionId);
            
            //Create Invoice if enabled
            if ($this->_common->getParamAutoInvoice() && $order->canInvoice())
            {
                $this->invoiceOrder($order);
            }
        }
        else
        {
            if ($order->getState() == 'pending_payment')
            {
                $order->cancel();
            }

            $historyComment = (
                \YottaPayIntegration\Payments\Model\Common::$historyCommentTransactionFailed
                . $yottaTransactionId
            );

            $order->addStatusHistoryComment($historyComment)
                ->setIsCustomerNotified(true)
                ->save();

            $order->save();
        }
    }
}
