<?php

namespace YottaPayIntegration\Payments\Api\Refund\Old;

/**
 * Class for process payment result from Yotta Pay API
 */
class Result implements ResultInterface
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var \Magento\Sales\Model\Order\Payment\Transaction\Builder
     */
    protected $_transactionBuilder;

    /**
     * @var \Magento\Sales\Model\Service\InvoiceService
     */
    protected $_invoiceService;

    /**
     * @var \Magento\Sales\Model\Order\CreditmemoFactory
     */
    protected $_creditmemoFactory;

    /**
     * @var \Magento\Sales\Model\Service\CreditmemoService
     */
    protected $_creditmemoService;

    /**
     * @var \Magento\Framework\DB\Transaction
     */
    protected $_dbTransaction;

    /**
     * @var \YottaPayIntegration\Payments\Model\Common
     */
    protected $_common;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Sales\Model\Order\CreditmemoFactory $creditmemoFactory
     * @param \Magento\Sales\Model\Order\Payment\Transaction\Builder $transactionBuilder
     * @param \Magento\Sales\Model\Service\InvoiceService $invoiceService
     * @param \Magento\Sales\Model\Service\CreditmemoService $creditmemoService
     * @param \Magento\Framework\DB\Transaction $dbTransaction
     * @param \YottaPayIntegration\Payments\Model\Common $common
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Magento\Sales\Model\Order\CreditmemoFactory $creditmemoFactory,
        \Magento\Sales\Model\Order\Payment\Transaction\Builder $transactionBuilder,
        \Magento\Sales\Model\Service\InvoiceService $invoiceService,
        \Magento\Sales\Model\Service\CreditmemoService $creditmemoService,
        \Magento\Framework\DB\Transaction $dbTransaction,
        \YottaPayIntegration\Payments\Model\Common $common
    ) {
        $this->_logger = $logger;
        $this->_transactionBuilder = $transactionBuilder;
        $this->_invoiceService = $invoiceService;
        $this->_creditmemoFactory = $creditmemoFactory;
        $this->_creditmemoService = $creditmemoService;
        $this->_dbTransaction = $dbTransaction;
        $this->_common = $common;
    }

    /**
     * Process refund result
     *
     * @api
     * @param string $type
     * @param string $customer_identifier
     * @param string $merchant_identifier
     * @param string $shop_transaction_identifier
     * @param string $yottapay_transaction_identifier
     * @param string $signature
     *
     * @return string
     */
    public function processRefundResult(
        $type,
        $customer_identifier,
        $merchant_identifier,
        $shop_transaction_identifier,
        $yottapay_transaction_identifier,
        $signature
    ) {
        try
        {
            $this->_logger->debug('YottaPay | processRefundResult | Start');

            $this->_logger->debug('YottaPay | processRefundResult | $type: ' . $type);
            $this->_logger->debug('YottaPay | processRefundResult | $customer_identifier: ' . $customer_identifier);
            $this->_logger->debug('YottaPay | processRefundResult | $merchant_identifier: ' . $merchant_identifier);
            $this->_logger->debug('YottaPay | processRefundResult | $shop_transaction_identifier: ' . $shop_transaction_identifier);
            $this->_logger->debug('YottaPay | processRefundResult | $yottapay_transaction_identifier: ' . $yottapay_transaction_identifier);
            $this->_logger->debug('YottaPay | processRefundResult | $signature: ' . $signature);


            $order = $this->checkRequestData(
                $type,
                $customer_identifier,
                $merchant_identifier,
                $shop_transaction_identifier,
                $yottapay_transaction_identifier
            );

            $this->checkRequestSign(
                $customer_identifier,
                $merchant_identifier,
                $shop_transaction_identifier,
                $yottapay_transaction_identifier,
                $signature
            );

            //Add refund transaction
            $this->addRefundTransactionToPayment($order, $yottapay_transaction_identifier);

            //Create credit memo
            $this->createCreditMemo($order);
            
            //Change order state and status
            $order->setState("closed")->setStatus("closed");
            $order->save();

            return 'OK';
        }
        catch (\Exception $e)
        {
            $errorMessage = (
                'Error processRefundResult Old [v. '
                . \YottaPayIntegration\Payments\Model\Common::$moduleVersion
                . ']: '
                . $e->getMessage()
            );

            $this->_logger->error('YottaPay | processRefundResult Old | ' . $errorMessage);

            throw new \Magento\Framework\Webapi\Exception(
                __($errorMessage),
                0,
                \Magento\Framework\Webapi\Exception::HTTP_INTERNAL_ERROR
            );
        }
        finally
        {
            $this->_logger->debug('YottaPay | processRefundResult Old | End');
        }
    }

    /**
     * Create credit memo for payment
     *
     * @param \Magento\Sales\Model\Order $order
     * @return void
     */
    private function createCreditMemo($order)
    {
        //Get invoice from order or create one if it doesn't exist
        $invoiceObj = null;

        foreach ($order->getInvoiceCollection() as $invoice)
        {
            $invoice->load($invoice->getId());
            $invoiceObj = $invoice;
        }

        if (is_null($invoiceObj))
        {
            $invoiceCanceled = $this->_invoiceService->prepareInvoice($order);
            $invoiceCanceled->register();
            $invoiceCanceled->pay();
            $invoiceCanceled->save();
            $transactionSave = $this->_dbTransaction->addObject($invoiceCanceled)->addObject($invoiceCanceled->getOrder());
            $transactionSave->save();

            foreach ($order->getInvoiceCollection() as $invoice)
            {
                $invoice->load($invoice->getId());
                $invoiceObj = $invoice;
            }
        }

        //Create Credit Memo
        $creditMemo = $this->_creditmemoFactory->createByInvoice($invoiceObj);
        $creditMemo->setOrder($order);
        $creditMemo->setInvoice($invoiceObj);

        foreach ($creditMemo->getAllItems() as $creditmemoItem)
        {   
            $creditmemoItem->setBackToStock(true);
        }

        $this->_creditmemoService->refund($creditMemo);
    }

    /**
     * Create refund transaction and add to payment
     *
     * @param \Magento\Sales\Model\Order $order
     * @param string $yottapay_transaction_identifier
     * @return void
     */
    private function addRefundTransactionToPayment($order, $yottapay_transaction_identifier)
    {
        //Prepare payment
        $message = __('Refunded amount of %1 online.', $order->getBaseCurrency()->formatTxt($order->getGrandTotal()));
        $paymentData = array();
        $refundTransactionId = 'Refund: ' . $yottapay_transaction_identifier;
        $payment = $order->getPayment();
        $payment->setLastTransId($refundTransactionId);
        $payment->setTransactionId($refundTransactionId);
        $payment->setAdditionalInformation([\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $paymentData]);
        $payment->setParentTransactionId($yottapay_transaction_identifier);
        $payment->setMessage($message);

        // Prepare transaction
        $transaction = $this->_transactionBuilder->setPayment($payment)
            ->setOrder($order)
            ->setTransactionId($refundTransactionId)
            ->setAdditionalInformation([\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $paymentData])
            ->setFailSafe(true)
            ->build(\Magento\Sales\Model\Order\Payment\Transaction::TYPE_REFUND);

        // Save payment and transaction
        $payment->save();
        $transaction->save();
    }

    /**
     * Check required fields, then return the appropriate order
     *
     * @param string $type
     * @param string $customer_identifier
     * @param string $merchant_identifier
     * @param string $shop_transaction_identifier
     * @param string $yottapay_transaction_identifier
     * @return \Magento\Sales\Model\Order
     */
    private function checkRequestData(
        $type,
        $customer_identifier,
        $merchant_identifier,
        $shop_transaction_identifier,
        $yottapay_transaction_identifier
    ) {
        $splitedShopTransactionIdentifier = explode('-', $shop_transaction_identifier);
        $orderIncrementId = end($splitedShopTransactionIdentifier);
        $order = $this->_common->getOrderByIncrementId($orderIncrementId);

        //Verify order status
        $orderStatus = $order->getState();
        if ($orderStatus != 'processing' && $orderStatus != 'complete')
        {
            throw new \Exception('Invalid order status for refund: ' . $orderStatus);
        }

        //Match source data
        if($customer_identifier == '0')
        {
            $source_customer_identifier = '0'; //ordered from guest
        }
        else
        {
            $source_customer_identifier = (empty($order->getCustomerId()) ? '0' : $order->getCustomerId());
        }

        $source_merchant_identifier = $this->_common->getParamMerchantId();
        $source_shop_transaction_identifier = (
            \YottaPayIntegration\Payments\Model\Common::$platformPrefix
            . '-'
            .  $source_customer_identifier
            . '-'
            . $order->getIncrementId()
        );
        $source_yottapay_transaction_identifier = $order->getPayment()->getLastTransId();

        if ($type != 'refundcompleted')
        {
            throw new \Exception('Invalid request data type: ' . $type);
        }

        if ($shop_transaction_identifier != $source_shop_transaction_identifier)
        {
            throw new \Exception(
                'Invalid request data shop_transaction_identifier: '
                . $shop_transaction_identifier . ' and ' . $source_shop_transaction_identifier
            );
        }

        if ($yottapay_transaction_identifier != $source_yottapay_transaction_identifier)
        {
            throw new \Exception(
                'Invalid request data yottapay_transaction_identifier: '
                . $yottapay_transaction_identifier . ' and ' . $source_yottapay_transaction_identifier
            );
        }

        if ($merchant_identifier != $source_merchant_identifier)
        {
            throw new \Exception(
                'Invalid request data merchant_identifier: '
                . $merchant_identifier . ' and ' . $source_merchant_identifier
            );
        }

        if ($customer_identifier != $source_customer_identifier)
        {
            throw new \Exception('Invalid request data customer_identifier: '
                . $customer_identifier . ' and ' . $source_customer_identifier
            );
        }

        return $order;
    }

    /**
     * Caluclate signature and compare with requested value
     *
     * @param string $customer_identifier
     * @param string $merchant_identifier
     * @param string $shop_transaction_identifier
     * @param string $yottapay_transaction_identifier
     * @param string $signature
     * @return void
     */
    private function checkRequestSign(
        $customer_identifier,
        $merchant_identifier,
        $shop_transaction_identifier,
        $yottapay_transaction_identifier,
        $signature
    ) {
        $merchantKey = $this->_common->getParamMerchantKey();

        $dataToSign = (
            $yottapay_transaction_identifier
            . $shop_transaction_identifier
            . $merchant_identifier
            . $customer_identifier
            . $merchantKey
        );

        $calculatedSignature = hash('sha256', $dataToSign);

        if ($signature != $calculatedSignature)
        {
            $this->_logger->debug('YottaPay | checkRequestSign | Signature is wrong: ' . $signature . ' and ' . $calculatedSignature);

            throw new \Exception('Signature is wrong');
        }
    }
}
