<?php

namespace YottaPayIntegration\Payments\Api\Refund;

/**
 * Class for process payment result from Yotta Pay API
 */
class Result implements ResultInterface
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var \Magento\Sales\Model\Order\Payment\Transaction\Builder
     */
    protected $_transactionBuilder;

    /**
     * @var \Magento\Sales\Model\Service\InvoiceService
     */
    protected $_invoiceService;

    /**
     * @var \Magento\Sales\Model\Order\CreditmemoFactory
     */
    protected $_creditmemoFactory;

    /**
     * @var \Magento\Sales\Model\Service\CreditmemoService
     */
    protected $_creditmemoService;

    /**
     * @var \Magento\Framework\DB\Transaction
     */
    protected $_dbTransaction;

    /**
     * @var \YottaPayIntegration\Payments\Model\Common
     */
    protected $_common;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Sales\Model\Order\CreditmemoFactory $creditmemoFactory
     * @param \Magento\Sales\Model\Order\Payment\Transaction\Builder $transactionBuilder
     * @param \Magento\Sales\Model\Service\InvoiceService $invoiceService
     * @param \Magento\Sales\Model\Service\CreditmemoService $creditmemoService
     * @param \Magento\Framework\DB\Transaction $dbTransaction
     * @param \YottaPayIntegration\Payments\Model\Common $common
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Magento\Sales\Model\Order\CreditmemoFactory $creditmemoFactory,
        \Magento\Sales\Model\Order\Payment\Transaction\Builder $transactionBuilder,
        \Magento\Sales\Model\Service\InvoiceService $invoiceService,
        \Magento\Sales\Model\Service\CreditmemoService $creditmemoService,
        \Magento\Framework\DB\Transaction $dbTransaction,
        \YottaPayIntegration\Payments\Model\Common $common
    ) {
        $this->_logger = $logger;
        $this->_transactionBuilder = $transactionBuilder;
        $this->_invoiceService = $invoiceService;
        $this->_creditmemoFactory = $creditmemoFactory;
        $this->_creditmemoService = $creditmemoService;
        $this->_dbTransaction = $dbTransaction;
        $this->_common = $common;
    }

    /**
     * Process refund result
     *
     * @api
     * @param string $orderId
     * @param string $userId
     *
     * @return string
     */
    public function processRefundResult($orderId, $userId)
    {
        try
        {
            $this->_logger->debug('YottaPay | processRefundResult | Start');

            $this->_logger->debug('YottaPay | processRefundResult | orderId: ' . $orderId);
            $this->_logger->debug('YottaPay | processRefundResult | userId: ' . $userId);

            $order = $this->checkRequestData($orderId, $userId);

            $yottaTransactionId = $order->getPayment()->getLastTransId();

            //Add refund transaction
            $this->addRefundTransactionToPayment($order, $yottaTransactionId);

            //Create credit memo
            $this->createCreditMemo($order);
            
            //Change order state and status
            $order->setState("closed")->setStatus("closed");
            $order->save();

            return 'OK';
        }
        catch (\Exception $e)
        {
            $errorMessage = (
                'Error processRefundResult [v. '
                . \YottaPayIntegration\Payments\Model\Common::$moduleVersion
                . ']: '
                . $e->getMessage()
            );

            $this->_logger->error('YottaPay | processRefundResult | ' . $errorMessage);

            throw new \Magento\Framework\Webapi\Exception(
                __($errorMessage),
                0,
                \Magento\Framework\Webapi\Exception::HTTP_INTERNAL_ERROR
            );
        }
        finally
        {
            $this->_logger->debug('YottaPay | processRefundResult | End');
        }
    }

    /**
     * Create credit memo for payment
     *
     * @param \Magento\Sales\Model\Order $order
     * @return void
     */
    private function createCreditMemo($order)
    {
        //Get invoice from order or create one if it doesn't exist
        $invoiceObj = null;

        foreach ($order->getInvoiceCollection() as $invoice)
        {
            $invoice->load($invoice->getId());
            $invoiceObj = $invoice;
        }

        if (is_null($invoiceObj))
        {
            $invoiceCanceled = $this->_invoiceService->prepareInvoice($order);
            $invoiceCanceled->register();
            $invoiceCanceled->pay();
            $invoiceCanceled->save();
            $transactionSave = $this->_dbTransaction->addObject($invoiceCanceled)->addObject($invoiceCanceled->getOrder());
            $transactionSave->save();

            foreach ($order->getInvoiceCollection() as $invoice)
            {
                $invoice->load($invoice->getId());
                $invoiceObj = $invoice;
            }
        }

        //Create Credit Memo
        $creditMemo = $this->_creditmemoFactory->createByInvoice($invoiceObj);
        $creditMemo->setOrder($order);
        $creditMemo->setInvoice($invoiceObj);

        foreach ($creditMemo->getAllItems() as $creditmemoItem)
        {   
            $creditmemoItem->setBackToStock(true);
        }

        $this->_creditmemoService->refund($creditMemo);
    }

    /**
     * Create refund transaction and add to payment
     *
     * @param \Magento\Sales\Model\Order $order
     * @param string $yottaTransactionId
     * @return void
     */
    private function addRefundTransactionToPayment($order, $yottaTransactionId)
    {
        //Prepare payment
        $message = __('Refunded amount of %1 online.', $order->getBaseCurrency()->formatTxt($order->getGrandTotal()));
        $paymentData = array();
        $refundTransactionId = 'Refund: ' . $yottaTransactionId;
        $payment = $order->getPayment();
        $payment->setLastTransId($refundTransactionId);
        $payment->setTransactionId($refundTransactionId);
        $payment->setAdditionalInformation([\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $paymentData]);
        $payment->setParentTransactionId($yottaTransactionId);
        $payment->setMessage($message);

        // Prepare transaction
        $transaction = $this->_transactionBuilder->setPayment($payment)
            ->setOrder($order)
            ->setTransactionId($refundTransactionId)
            ->setAdditionalInformation([\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $paymentData])
            ->setFailSafe(true)
            ->build(\Magento\Sales\Model\Order\Payment\Transaction::TYPE_REFUND);

        // Save payment and transaction
        $payment->save();
        $transaction->save();
    }

    /**
     * Check required fields, then return the appropriate order
     *
     * @param string $orderId
     * @param string $userId
     * @return \Magento\Sales\Model\Order
     */
    private function checkRequestData($orderId, $userId)
    {
        $sourceUserId = $this->_common->getUserId();

        if (empty($sourceUserId))
        {
            throw new \Exception('Empty user_id config param.');
        }

        if ($userId != $sourceUserId)
        {
            throw new \Exception('Unknown userId.');
        }

        $splitedShopTransactionIdentifier = explode('-', $orderId);
        $orderIncrementId = end($splitedShopTransactionIdentifier);
        $order = $this->_common->getOrderByIncrementId($orderIncrementId);

        //Verify order status
        $orderStatus = $order->getState();
        if ($orderStatus != 'processing' && $orderStatus != 'complete')
        {
            throw new \Exception('Invalid order status for refund: ' . $orderStatus);
        }

        return $order;
    }
}
