<?php

namespace YottaPayIntegration\Payments\Controller\CreateDeferred;

/**
 * Class for create loyalty deferred intent.
 */
class Index extends \Magento\Framework\App\Action\Action
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $_customerSession;

    /**
     * @var \Magento\Framework\Controller\Result\JsonFactory
     */
    private $_resultJsonFactory;

    /**
     * @var \Magento\Framework\Message\ManagerInterface
     */
    protected $_messageManager;

    /**
     * @var \YottaPayIntegration\Payments\Model\Common
     */
    protected $_common;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     * @param \YottaPayIntegration\Payments\Model\Common $common
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\App\Action\Context $context,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        \YottaPayIntegration\Payments\Model\Common $common
    ) {
        $this->_logger = $logger;
        $this->_resultJsonFactory = $resultJsonFactory;
        $this->_customerSession = $customerSession;
        $this->_messageManager = $messageManager;
        $this->_common = $common;

        return parent::__construct($context);
    }

    /**
     * Create loyalty deferred intent and return response
     *
     * @return {}
     */
    public function execute()
    {
        try
        {
            $this->_logger->debug('YottaPay | CreateDeferred execute | Start');

            //Initialize for response
            $resultJson = $this->_resultJsonFactory->create();

            //Default response
            $result = array(
                'status' => '0',
                'link' => '',
                'error' => 'Request failed. Please contact the store owner to solve the problem.'
            );

            //Verify session
            if (!($this->_customerSession->isLoggedIn()))
            {
                $result['error']= 'Sign in with your email address';
                throw new \Exception('Session is not logged in.');
            }

            //Verify plugin enabled
            if (!($this->_common->getParamActive()))
            {
                throw new \Exception('Payment method is disabled in admin panel.');
            }

            //Verify loyalty bonus
            if ($this->_common->getParamLoyaltyBonus() == 0)
            {
                throw new \Exception('Loyalty Bonus is disabled in admin panel.');
            }

            //Validate request data
            $order = $this->validateLoyaltyRequestData();

            if ($this->_common->usedOldApiKeys())
            {
                //Perform request to Yotta Pay API
                $responseData = $this->_common->performPostRequest(
                    \YottaPayIntegration\Payments\Model\Common::$createDeferredEndpointOld,
                    json_encode($this->getDataToSendOld($order))
                );

                //Process response data from Yotta Pay API
                $result = $this->processResponseDataOld($responseData);
            }
            else
            {
                //Perform request to Yotta Pay API
                $responseData = $this->_common->performPostRequest(
                    \YottaPayIntegration\Payments\Model\Common::$createDeferredEndpoint,
                    json_encode($this->getDataToSend($order))
                );

                //Process response data from Yotta Pay API
                $result = $this->processResponseData($responseData);
            }
        }
        catch (\Exception $e)
        {
            $this->_logger->error('YottaPay | CreateDeferred execute | ' . $e->getMessage());
            $this->_messageManager->addError($result['error']);
        }
        finally
        {
            $this->_logger->debug('YottaPay | CreateDeferred execute | End');
            return $resultJson->setData($result);
        }
    }

    /**
     * Process a response data
     *
     * @param array $responseData
     * @return array
     */
    private function processResponseDataOld($responseData)
    {
        if (!array_key_exists('link', $responseData))
        {
            throw new \Exception('processResponseDataOld: Missing required parameters in Yotta Pay API response.');
        }

        $result = array(
            'status' => '1',
            'link' => $responseData['link'],
            'error' => ''
        );

        return $result;
    }

    /**
     * Process a response data
     *
     * @param array $responseData
     * @return array
     */
    private function processResponseData($responseData)
    {
        if (!array_key_exists('result', $responseData))
        {
            throw new \Exception('processResponseData: Missing required parameters in Yotta Pay API response.');
        }

        $result = array(
            'status' => '1',
            'link' => $responseData['result'],
            'error' => ''
        );

        return $result;
    }

    /**
     * Get request parameters, validate data and return order
     *
     * @return \Magento\Sales\Model\Order $order
     */
    private function validateLoyaltyRequestData()
    {
         $postData = $this->_common->getRequestData();

        //Verify required parameters
        if (!isset($postData['order_id']))
        {
            throw new \Exception('Missing required parameters.');
        }

        //Verify order
        $orderId = $postData['order_id'];
        $order = $this->_common->getOrderById($orderId);

        //Verify order status
        $orderStatus = $order->getState();
        if ($orderStatus != 'processing' && $orderStatus != 'complete')
        {
            throw new \Exception('Invalid order status to create deferred: ' . $orderStatus);
        }

        //Compare order created date and loyalty_start_period option
        if (!($this->_common->validateOrderDateMatchLoyalty($order)))
        {
            throw new \Exception('Too old order.');
        }

        return $order;
    }

    /**
     * Create data to create deferred request
     *
     * @param \Magento\Sales\Model\Order $order
     * @return array
     */
    private function getDataToSendOld($order)
    {
        $baseUrl = $this->_common->getBaseUrl();

        $merchantId = $this->_common->getParamMerchantId();
        $merchantKey = $this->_common->getParamMerchantKey();

        //Get clientUserId
        $clientUserId = $this->_common->getClientUserId($order);

        //Get loyalty amount
        $loyaltyAmount = $this->_common->getLoyaltyAmount($order);

        $body = [
            'type' => 'accrual',
            'merchantIdentifier' => $merchantId,
            'clientUserId' => $clientUserId,
            'amount' => $loyaltyAmount,
            'comment' => $order->getId(),
            'notificationUrl' => (
                $baseUrl
                . \YottaPayIntegration\Payments\Model\Common::$apiLoyaltyResultOld
            ),
            'signature' => ''
        ];

        $dataToSign = (
            $body['merchantIdentifier']
            . $body['clientUserId']
            . $body['amount']
            . $body['comment']
            . $body['notificationUrl']
            . $merchantKey
        );

        $sign = hash('sha256', $dataToSign);

        $body['signature'] = $sign;

        $this->_logger->debug('YottaPay | CreateDeferred getDataToSendOld | body: ' . print_r($body, TRUE));

        return $body;
    }

    /**
     * Create data to create deferred request
     *
     * @param \Magento\Sales\Model\Order $order
     * @return array
     */
    private function getDataToSend($order)
    {
        $userToken = $this->_common->getUserToken();
        $clientUserId = $this->_common->getClientUserId($order);
        $loyaltyAmount = $this->_common->getLoyaltyAmount($order);

        $body = [
            'userAccessToken' => $userToken,
            'customerId' => $clientUserId,
            'amount' => $loyaltyAmount,
            'comment' => $order->getId(),
        ];

        $this->_logger->debug('YottaPay | CreateDeferred getDataToSend | body: ' . print_r($body, TRUE));

        return $body;
    }
}
