<?php

namespace YottaPayIntegration\Payments\Controller\PaymentIntent;

/**
 * Class for create payment intent and redirect to the Yotta Pay payment page.
 */
class Index extends \Magento\Framework\App\Action\Action
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $_checkoutSession;

    /**
     * @var \Magento\Framework\Controller\ResultFactory
     */
    protected $_resultFactory;

    /**
     * @var \Magento\Framework\Message\ManagerInterface
     */
    protected $_messageManager;

    /**
     * @var \YottaPayIntegration\Payments\Model\Common
     */
    protected $_common;


    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Framework\Controller\ResultFactory $resultFactory
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     * @param \YottaPayIntegration\Payments\Model\Common $common
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\App\Action\Context $context,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Framework\Controller\ResultFactory $resultFactory,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        \YottaPayIntegration\Payments\Model\Common $common
    ) {
        $this->_logger = $logger;
        $this->_resultFactory = $resultFactory;
        $this->_checkoutSession = $checkoutSession;
        $this->_messageManager = $messageManager;
        $this->_common = $common;

        return parent::__construct($context);
    }

    /**
     * Create payment intent and redirect to the Yotta Pay payment page
     *
     * @return \Magento\Framework\Controller\Result\RedirectFactory
     */
    public function execute()
    {
        try 
        {
            $this->_logger->debug('YottaPay | PaymentIntent execute | Start');

            //Initialize for response
            $redirect = $this->_resultFactory->create(\Magento\Framework\Controller\ResultFactory::TYPE_REDIRECT);
            $redirectUrl = $this->_common->getBaseUrl() . 'checkout/cart';
    
            //Try to get and validate order
            $order = $this->_checkoutSession->getLastRealOrder();
            $this->validateOrder($order);

            if ($this->_common->usedOldApiKeys())
            {
                //Perform request to Yotta Pay API
                $responseData = $this->_common->performPostRequest(
                    \YottaPayIntegration\Payments\Model\Common::$createPaymentIntentEndpointOld,
                    json_encode($this->getDataToSendOld($order))
                );

                //Process response data from Yotta Pay API
                $redirectUrl = $this->processResponseDataOld($responseData, $order);
            }
            else
            {
                //Perform request to Yotta Pay API
                $responseData = $this->_common->performPostRequest(
                    \YottaPayIntegration\Payments\Model\Common::$createPaymentIntentEndpoint,
                    json_encode($this->getDataToSend($order))
                );

                //Process response data from Yotta Pay API
                $redirectUrl = $this->processResponseData($responseData, $order);
            }
        }
        catch (\YottaPayIntegration\Payments\Exception\YottaPayException $e)
        {
            $this->_logger->error('YottaPay | PaymentIntent execute | ' . $e->getMessage());

            $this->_checkoutSession->restoreQuote();
        }
        catch (\Exception $e)
        {
            $this->_logger->error('YottaPay | PaymentIntent execute | ' . $e->getMessage());

            $this->_checkoutSession->restoreQuote();

            $this->_messageManager->addError(
                'Payment was failed. '
                . 'Please contact the store owner to solve the problem.'
            );
        }
        finally
        {
            $this->_logger->debug('YottaPay | PaymentIntent execute | End');
            $redirect->setUrl($redirectUrl);
            return $redirect;
        }
    }

    /**
     * Create data to payment intent request
     *
     * @param \Magento\Sales\Model\Order $order
     * @return array
     */
    private function getDataToSendOld($order)
    {
        $baseUrl = $this->_common->getBaseUrl();

        $body = [
                'type' => 'creation',
                'shop_transaction_identifier' => (
                    \YottaPayIntegration\Payments\Model\Common::$platformPrefix
                    . '-'
                    . (empty($order->getCustomerId()) ? '0' : $order->getCustomerId())
                    . '-'
                    . $order->getIncrementId()
                ),
                'merchant_identifier' => $this->_common->getParamMerchantId(),
                'customer_identifier' => (empty($order->getCustomerId()) ? '0' : $order->getCustomerId()),
                'amount' =>  number_format(round($order->getGrandTotal(), 2, PHP_ROUND_HALF_UP), 2, '.', ''),
                'currency' => $order->getOrderCurrencyCode(),
                'cancellation_url' => (
                    $baseUrl
                    . \YottaPayIntegration\Payments\Model\Common::$apiPaymentCancellation
                    . '?orderIncrementId='
                    . $order->getIncrementId()
                ),
                'url_payment_result' => (
                    $baseUrl
                    . \YottaPayIntegration\Payments\Model\Common::$apiPaymentResultOld
                ),
                'url_merchant_page_success' => $baseUrl . 'checkout/onepage/success',
                'url_merchant_page_cancel' => $baseUrl . 'checkout/onepage/failure',
                'yotta_notification_id' => '',
                'signature' => ''
        ];

        $dataToSign = (
            $body['shop_transaction_identifier']
            . $body['merchant_identifier']
            . $body['customer_identifier']
            . $body['amount']
            . $body['currency']
            . $body['url_payment_result']
            . $body['url_merchant_page_success']
            . $body['url_merchant_page_cancel']
            . $body['cancellation_url']
            . $this->_common->getParamMerchantKey()
        );

        $sign = hash('sha256', $dataToSign);
        
        $body['signature'] = $sign;

        $this->_logger->debug('YottaPay | PaymentIntent getDataToSendOld | body: ' . print_r($body, TRUE));

        return $body;
    }

    /**
     * Create data to payment intent request
     *
     * @param \Magento\Sales\Model\Order $order
     * @return array
     */
    private function getDataToSend($order)
    {
        $userToken = $this->_common->getUserToken();
        $gtmId = $this->_common->getGtmId(); 
        $baseUrl = $this->_common->getBaseUrl();
        
        $customerId = empty($order->getCustomerId()) ? '0' : $order->getCustomerId();
        $transactionId = (
            \YottaPayIntegration\Payments\Model\Common::$platformPrefix
            . '-'
            . $customerId
            . '-'
            . $order->getIncrementId()
        );
        $reference = $transactionId;
        $amount = number_format(round($order->getGrandTotal(), 2, PHP_ROUND_HALF_UP), 2, '.', '');        
        $webhookUrl = $baseUrl . \YottaPayIntegration\Payments\Model\Common::$apiPaymentResult;
        $redirectUrlSuccess = $baseUrl . 'checkout/onepage/success';
        $redirectUrlFail = $baseUrl . 'checkout/onepage/failure';
        $cancellationUrl = (
            $baseUrl
            . \YottaPayIntegration\Payments\Model\Common::$apiPaymentCancellation
            . '?orderIncrementId='
            . $order->getIncrementId()
        );

        $body = [
            'userAccessToken' => $userToken,
            'transactionId' => $transactionId,
            'reference' => $reference,
            'amount' => $amount,
            'customerId' => $customerId,
            'redirectUrlSuccess' => $redirectUrlSuccess,
            'redirectUrlFail' => $redirectUrlFail,
            'cancellationUrl' => $cancellationUrl,
            'webhookUrl' => $webhookUrl,
        ];

        if (!empty($gtmId))
        {
            $body['tagmanagerId'] = $gtmId;
            $body['otherRequestParams'] = [
                'utm_source' => '',
                'utm_medium' => '',
                'utm_campaign' => '',
                'utm_content' => '',
                'utm_term' => '',
            ];

            try 
            {
                //Sourcebuster
                $sbParams = [];
                $sbCookieValue = $_COOKIE['sbjs_first'];

                if (isset($sbCookieValue) && !empty($sbCookieValue))
                {
                    $splitedSbCookieValue = explode('|||', $sbCookieValue);
                    foreach ($splitedSbCookieValue as &$value)
                    {
                        $splitedValue = explode('=', $value);
                        $sbParams[$splitedValue[0]] = $splitedValue[1];
                    }
                
                    $body['otherRequestParams'] = [
                        'utm_source' => $sbParams['src'],
                        'utm_medium' => $sbParams['mdm'],
                        'utm_campaign' => $sbParams['cmp'],
                        'utm_content' => $sbParams['cnt'],
                        'utm_term' => $sbParams['trm'],
                    ];
                }
                else
                {
                    throw new \Exception('No cookie.');
                }
            }
            catch (\Exception $e)
            {
                $this->_logger->error('YottaPay | PaymentIntent getDataToSend | Sourcebuster cookie exception');
            }
        }

        $this->_logger->debug('YottaPay | PaymentIntent getDataToSend | body: ' . print_r($body, TRUE));

        return $body;
    }

    /**
     * Validate order
     *
     * @param \Magento\Sales\Model\Order $order
     * @return bool
     */
    private function validateOrder($order)
    {
        if ($order->getState() != 'pending_payment') 
        {
            $this->_messageManager->addError(
                'Looks like the current order has already been paid:'
                . ' you can check paid orders on the My Orders page.'
            );
            throw new \YottaPayIntegration\Payments\Exception\YottaPayException(
                'validateOrderState exception -- Invalid order status for payment intent (#'
                . $order->getIncrementId() . '): "'. $order->getState() . '".'
            );
        }

        if (!($this->_common->validateBillingCountry($order)))
        {
            $this->_messageManager->addError(
                'Sorry, Yotta Pay currently is only available in the'
                . ' United Kingdom.'
            );
            throw new \YottaPayIntegration\Payments\Exception\YottaPayException('validateBillingCountry exception.');
        }

        if (!($this->_common->validateOrderCurrency($order)))
        {
            $this->_messageManager->addError('Sorry, Yotta Pay currently works only with GBP.');
            throw new \YottaPayIntegration\Payments\Exception\YottaPayException('validateOrderCurrency exception.');
        }

        if (!($this->_common->validateMinOrderAmount($order)))
        {
            $this->_messageManager->addError(
                'Sorry, the minimum allowed order total is'
                . ' 0.01 GBP to use Yotta Pay payment method.'
            );
            throw new \YottaPayIntegration\Payments\Exception\YottaPayException('validateMinOrderAmount exception.');
        }
    }

    /**
     * Process a response data
     *
     * @param array $responseData
     * @param \Magento\Sales\Model\Order $order
     * @return string
     */
    private function processResponseDataOld($responseData, $order)
    {
        if (!array_key_exists('url_process_payment_intent', $responseData))
        {
            throw new \Exception('Missing required parameters in Yotta Pay API response.');
        }

        $order->addStatusHistoryComment(
            \YottaPayIntegration\Payments\Model\Common::$historyCommentPaymentIntent
            . $responseData['url_process_payment_intent']
        )
            ->setIsCustomerNotified(true)
            ->save();

        $order->save();

        return $responseData['url_process_payment_intent'];
    }

    /**
     * Process a response data
     *
     * @param array $responseData
     * @param \Magento\Sales\Model\Order $order
     * @return string
     */
    private function processResponseData($responseData, $order)
    {
        if (!array_key_exists('url', $responseData))
        {
            throw new \Exception('Missing required parameters in Yotta Pay API response.');
        }

        $order->addStatusHistoryComment(
            \YottaPayIntegration\Payments\Model\Common::$historyCommentPaymentIntent
            . $responseData['url']
        )
            ->setIsCustomerNotified(true)
            ->save();

        $order->save();

        return $responseData['url'];
    }
}
