<?php

namespace YottaPayIntegration\Payments\Controller\RefundIntent;

/**
 * Class for create payment intent and redirect to the Yotta Pay payment page.
 */
class Index extends \Magento\Framework\App\Action\Action
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $_customerSession;

    /**
     * @var \Magento\Framework\Data\Form\FormKey\Validator
     */
    protected $_formKeyValidator;

    /**
     * @var \Magento\Framework\Controller\Result\JsonFactory
     */
    private $_resultJsonFactory;

    /**
     * @var \Magento\Framework\Message\ManagerInterface
     */
    protected $_messageManager;

    /**
     * @var \Magento\Framework\Notification\NotifierInterface
     */
    protected $_notifierPool;

    /**
     * @var \YottaPayIntegration\Payments\Model\Common
     */
    protected $_common;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Framework\Data\Form\FormKey\Validator $formKeyValidator
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     * @param \Magento\Framework\Notification\NotifierInterface $notifierPool
     * @param \YottaPayIntegration\Payments\Model\Common $common
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\App\Action\Context $context,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Framework\Data\Form\FormKey\Validator $formKeyValidator,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        \Magento\Framework\Notification\NotifierInterface $notifierPool,
        \YottaPayIntegration\Payments\Model\Common $common
    ) {
        $this->_logger = $logger;
        $this->_resultJsonFactory = $resultJsonFactory;
        $this->_customerSession = $customerSession;
        $this->_formKeyValidator = $formKeyValidator;
        $this->_messageManager = $messageManager;
        $this->_notifierPool = $notifierPool;
        $this->_common = $common;

        return parent::__construct($context);
    }

    /**
     * Create refund intent and return response status
     *
     * @return {}
     */
    public function execute()
    {
        try
        {
            $this->_logger->debug('YottaPay | RefundIntent execute | Start');

            //Initialize for response
            $resultJson = $this->_resultJsonFactory->create();

            //Default response
            $result = array(
                'status' => '0',
                'error' => 'Request failed. Please contact the store owner to solve the problem.'
            );

            //Verify session
            if (!($this->_customerSession->isLoggedIn()))
            {
                $result['error']= 'Sign in with your email address';
                throw new \Exception('Session is not logged in.');
            }

            //Validate request data
            $validatedData = $this->validateRefundRequestData();
            $order = $validatedData['order'];

            //Validate available length
            $this->validateAvailableLength($validatedData);

            //Validate Account number and Sort code fields
            if (!($this->validateNumeric($validatedData)))
            {
                $result['error']= 'Incorrect Account number or Sort code';
                throw new \Exception('Incorrect Account number or Sort code.');
            }

            if ($this->_common->usedOldApiKeys())
            {
                //Perform request to Yotta Pay API
                $responseData = $this->_common->performPostRequest(
                    \YottaPayIntegration\Payments\Model\Common::$crateRefundIntentEndpointOld,
                    json_encode($this->getDataToSendOld($order, $validatedData))
                );

                //Process response data from Yotta Pay API
                $result = $this->processResponseDataOld($responseData, $order);
            }
            else
            {
                //Perform request to Yotta Pay API
                $responseData = $this->_common->performPostRequest(
                    \YottaPayIntegration\Payments\Model\Common::$crateRefundIntentEndpoint,
                    json_encode($this->getDataToSend($order, $validatedData))
                );

                //Process response data from Yotta Pay API
                $result = $this->processResponseData($responseData, $order);
            }
        }
        catch (\Exception $e)
        {
            $this->_logger->error('YottaPay | RefundIntent execute | ' . $e->getMessage());

            if (isset($responseData))
            {
                $result['error'] = $responseData['error_message'];
                $this->_logger->error('YottaPay | RefundIntent execute | response_data error_message: ' . $responseData['error_message']);
            }

            $this->_messageManager->addError($result['error']);
        }
        finally
        {
            $this->_logger->debug('YottaPay | RefundIntent execute | End');
            return $resultJson->setData($result);
        }
    }

    /**
     * Process a response data
     *
     * @param array $responseData
     * @param \Magento\Sales\Model\Order $order
     * @return array
     */
    private function processResponseDataOld($responseData, $order)
    {
        if (!array_key_exists('ok', $responseData))
        {
            throw new \Exception('processResponseDataOld: Missing required parameters in Yotta Pay API response.');
        }

        //Validate ok status
        $resposeStatus = $responseData['ok'];
        if (!(boolval($resposeStatus) && strtolower($resposeStatus) != 'false'))
        {
            throw new \Exception('processResponseDataOld: Request has been failed.');
        }

        //Add history comment
        $order->addStatusHistoryComment(\YottaPayIntegration\Payments\Model\Common::$historyCommentRefundRequested)
            ->setIsCustomerNotified(true)
            ->save();
        $order->save();

        try
        {
            //Add customer notice
            $this->_messageManager->addSuccessMessage('Request has been submitted successfully');

            //Add admin notice
            $msgArg1 = $order->getIncrementId();
            $msgArg2 = (
                \YottaPayIntegration\Payments\Model\Common::$platformPrefix
                . '-'
                . (empty($order->getCustomerId()) ? '0' : $order->getCustomerId())
                . '-'
                . $order->getIncrementId()
            );
            $title = __('Yotta Pay refund requested for order #%1', $msgArg1);
            $message = __('Refund : %1.', $msgArg2);
            $this->_notifierPool->addNotice($title, $message);
        }
        catch (\Exception $e)
        {
            $this->_logger->error('YottaPay | processResponseData notice | ' . $e->getMessage());
        }

        $result = array(
            'status' => '1',
            'error' => ''
        );

        return $result;
    }

    /**
     * Process a response data
     *
     * @param array $responseData
     * @param \Magento\Sales\Model\Order $order
     * @return array
     */
    private function processResponseData($responseData, $order)
    {
        if (!array_key_exists('success', $responseData))
        {
            throw new \Exception('Missing required parameters in Yotta Pay API response.');
        }

        //Validate success status
        $resposeStatus = $responseData['success'];
        if (!(boolval($resposeStatus) && strtolower($resposeStatus) != 'false'))
        {
            throw new \Exception('Request has been failed. It is possible that a refund request for this order has already been sent previously.');
        }

        //Add history comment
        $order->addStatusHistoryComment(\YottaPayIntegration\Payments\Model\Common::$historyCommentRefundRequested)
            ->setIsCustomerNotified(true)
            ->save();
        $order->save();

        try
        {
            //Add customer notice
            $this->_messageManager->addSuccessMessage('Request has been submitted successfully');

            //Add admin notice
            $msgArg1 = $order->getIncrementId();
            $msgArg2 = (
                \YottaPayIntegration\Payments\Model\Common::$platformPrefix
                . '-'
                . (empty($order->getCustomerId()) ? '0' : $order->getCustomerId())
                . '-'
                . $order->getIncrementId()
            );
            $title = __('Yotta Pay refund requested for order #%1', $msgArg1);
            $message = __('Refund : %1.', $msgArg2);
            $this->_notifierPool->addNotice($title, $message);
        }
        catch (\Exception $e)
        {
            $this->_logger->error('YottaPay | processResponseData notice | ' . $e->getMessage());
        }

        $result = array(
            'status' => '1',
            'error' => ''
        );

        return $result;
    }

    /**
     * Validate refund form data
     *
     * @return array
     */
    private function validateRefundRequestData()
    {       
        $postData = $this->getRequest()->getPostValue();

        //Verify form key
        if (!($this->_formKeyValidator->validate($this->getRequest())))
        {
            throw new \Exception('Invalid form key.');
        }

        //Verify required parameters
        if (!isset($postData['order_id']) ||
            !isset($postData['refund_receiver_full_name']) ||
            !isset($postData['refund_receiver_account_number']) ||
            !isset($postData['refund_receiver_sort_code']) ||
            !isset($postData['refund_receiver_comment']))
        {
            throw new \Exception('Missing required parameters.');
        }

        //Verify order
        $orderId = $postData['order_id'];
        $order = $this->_common->getOrderById($orderId);

        //Verify order status
        $orderStatus = $order->getState();
        if ($orderStatus != 'processing' && $orderStatus != 'complete')
        {
            throw new \Exception('Invalid order status for refund: ' . $orderStatus);
        }

        //Verify customer
        $tempCustomerId = (empty($order->getCustomerId()) ? '0' : $order->getCustomerId());
        if ($tempCustomerId != $this->_customerSession->getCustomer()->getId())
        {
            throw new \Exception('Another customer.');
        }

        $sanitizedData = array(
            'order' => $order,
            'refund_receiver_full_name' => trim($postData['refund_receiver_full_name']),
            'refund_receiver_account_number' => trim($postData['refund_receiver_account_number']),
            'refund_receiver_sort_code' => trim($postData['refund_receiver_sort_code']),
            'refund_receiver_comment' => trim($postData['refund_receiver_comment'])
        );

        return $sanitizedData;
    }

    /**
     * Validate available length of input fields
     *
     * @param array $validatedData
     */
    private function validateAvailableLength($validatedData)
    {
        if (strlen($validatedData['refund_receiver_full_name']) == 0 ||
            strlen($validatedData['refund_receiver_full_name']) > 64)
        {
            throw new \Exception('Account holder name is required field (length up to 64 characters).');
        }

        if (strlen($validatedData['refund_receiver_account_number']) != 8)
        {
            throw new \Exception('Account number is required field (length 8 characters).');
        }

        if (strlen($validatedData['refund_receiver_sort_code']) != 6)
        {
            throw new \Exception('Sort code is required field (length 6 characters).');
        }

        if (strlen($validatedData['refund_receiver_comment']) == 0 ||
            strlen($validatedData['refund_receiver_comment']) > 1000)
        {
            throw new \Exception('Comment is required field (length up to 1000 characters).');
        }
    }

    /**
     * Validate values of Account number and Sort code fields
     *
     * @param array $validatedData
     * @return bool
     */
    private function validateNumeric($validatedData)
    {
        $accountNumberCleared = preg_replace( '/[^0-9]/', '', $validatedData['refund_receiver_account_number'] );
        $sortCodeCleared = preg_replace( '/[^0-9]/', '', $validatedData['refund_receiver_sort_code'] );
        
        if ((strlen($accountNumberCleared) != 8) || (strlen($sortCodeCleared) != 6))
        {
            return false;
        }

        return true;
    }

    /**
     * Create data to refund intent request
     *
     * @param \Magento\Sales\Model\Order $order
     * @param array $validatedData
     * @return array
     */
    private function getDataToSendOld($order, $validatedData)
    {
        $baseUrl = $this->_common->getBaseUrl();

        $merchantId = $this->_common->getParamMerchantId();
        $merchantKey = $this->_common->getParamMerchantKey();

        $body = [
                'type' => 'refund',
                'yottapay_transaction_identifier' => $order->getPayment()->getLastTransId(),
                'merchant_identifier' => $merchantId,               
                'url_payment_refund_result' => (
                    $baseUrl
                    . \YottaPayIntegration\Payments\Model\Common::$apiRefundResultOld
                ),               
                'refund_receiver_full_name' => $validatedData['refund_receiver_full_name'],
                'refund_receiver_account_number' => $validatedData['refund_receiver_account_number'],
                'refund_receiver_sort_code' => $validatedData['refund_receiver_sort_code'],
                'refund_receiver_comment' => $validatedData['refund_receiver_comment'],
                'signature' => ''
        ];

        $dataToSign = (
            $body['merchant_identifier']
            . $body['url_payment_refund_result']
            . $body['yottapay_transaction_identifier']
            . $body['refund_receiver_full_name']
            . $body['refund_receiver_account_number']
            . $body['refund_receiver_sort_code']
            . $body['refund_receiver_comment']
            . $merchantKey
        );

        $sign = hash('sha256', $dataToSign);
        
        $body['signature'] = $sign;

        $this->_logger->debug('YottaPay | RefundIntent getDataToSendOld | body: ' . print_r($body, TRUE));

        return $body;
    }

    /**
     * Create data to refund intent request
     *
     * @param \Magento\Sales\Model\Order $order
     * @param array $validatedData
     * @return array
     */
    private function getDataToSend($order, $validatedData)
    {
        $baseUrl = $this->_common->getBaseUrl();
        $userToken = $this->_common->getUserToken();

        $yottaTransactionId = $order->getPayment()->getLastTransId();
        $customerId = empty($order->getCustomerId()) ? '0' : $order->getCustomerId();
        $externalOrderId = (
            \YottaPayIntegration\Payments\Model\Common::$platformPrefix
            . '-'
            . $customerId
            . '-'
            . $order->getIncrementId()
        );
        $webhookUrl = $baseUrl . \YottaPayIntegration\Payments\Model\Common::$apiRefundResult;
        $receiverFullName = $validatedData['refund_receiver_full_name'];
        $receiverAccountNumber = $validatedData['refund_receiver_account_number'];
        $receiverSortCode = $validatedData['refund_receiver_sort_code'];
        $reason = $validatedData['refund_receiver_comment'];

        $body = [
            'userAccessToken' => $userToken,
            'yottaTransactionId' => $yottaTransactionId,
            'externalOrderId' => $externalOrderId,
            'webhookUrl' => $webhookUrl,
            'receiverFullName' => $receiverFullName,
            'receiverAccountNumber' => $receiverAccountNumber,
            'receiverSortCode' => $receiverSortCode,
            'reason' => $reason,
        ];

        $this->_logger->debug('YottaPay | RefundIntent getDataToSend | body: ' . print_r($body, TRUE));

        return $body;
    }
}
