<?php

namespace YottaPayIntegration\Payments\Model\Config\Backend;

/**
 * 
 */
class LoyaltyBonusOption extends \Magento\Framework\App\Config\Value
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \Magento\Framework\Message\ManagerInterface
     */
    protected $_messageManager;

    /**
     * @var \YottaPayIntegration\Payments\Model\Common
     */
    protected $_common;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     * @param \YottaPayIntegration\Payments\Model\Common $common
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $config
     * @param \Magento\Framework\App\Cache\TypeListInterface $cacheTypeList
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        \YottaPayIntegration\Payments\Model\Common $common,
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\App\Config\ScopeConfigInterface $config,
        \Magento\Framework\App\Cache\TypeListInterface $cacheTypeList,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = [] 
    ) {
        $this->_logger = $logger;
        $this->_messageManager = $messageManager;
        $this->_common = $common;

        parent::__construct($context, $registry, $config, $cacheTypeList, $resource, $resourceCollection, $data);
    }

    /**
     * Verify merchant loyalty programm status before save loyalty_bonus field value
     *
     * @return \Magento\Framework\Model\AbstractModel
     */
    public function beforeSave()
    {
        try
        {
            $this->_logger->debug('YottaPay | LoyaltyBonusOption beforeSave | Start');
            
            if ($this->_common->usedOldApiKeys())
            {
                $merchantId = $this->_common->getParamMerchantId();
                $merchantKey = $this->_common->getParamMerchantKey();

                if (empty($merchantId) || empty($merchantKey))
                {
                    $this->setValue(0);
                    return parent::beforeSave();
                }
            
                $value = (int)$this->getValue();

                if ($value > 0)
                {
                    //Perform request to Yotta Pay API
                    $responseData = $this->_common->performPostRequest(
                        \YottaPayIntegration\Payments\Model\Common::$loyaltyIsEnabledEndpointOld,
                        json_encode($this->getDataToSendOld($merchantId, $merchantKey))
                    );

                    //Process response data from Yotta Pay API
                    $this->processResponseData($responseData);
                }
            }
            else
            {
                $userToken = $this->_common->getUserToken();

                if (empty($userToken))
                {
                    $this->setValue(0);
                    return parent::beforeSave();
                }
            
                $value = (int)$this->getValue();

                if ($value > 0)
                {
                    //Perform request to Yotta Pay API
                    $responseData = $this->_common->performGetRequest(
                        \YottaPayIntegration\Payments\Model\Common::$loyaltyIsEnabledEndpoint . '/' . $userToken
                    );

                    //Process response data from Yotta Pay API
                    $this->processResponseData($responseData);
                }
            }
        }
        catch (\Exception $e)
        {
            $this->_logger->error('YottaPay | LoyaltyBonusOption beforeSave | ' . $e->getMessage());

            $errMsg = (
                'To enable Loyalty Bonus check that the Loyalty program has been enabled in the YottaPay App.'
                . ' Check that &quot;YottaPay App > Business > Loyalty program > Enable loyalty program&quot;,'
                . ' has been enabled and set accordingly.'
            );

            $this->_messageManager->addError($errMsg);

            $this->setValue(0);
        }
        finally
        {
            $this->_logger->debug('YottaPay | LoyaltyBonusOption beforeSave | End');
            return parent::beforeSave();
        }
    }

    /**
     * Process a response data
     *
     * @param array $responseData
     */
    private function processResponseData($responseData)
    {
        if (!array_key_exists('result', $responseData))
        {
            throw new \Exception('Missing required parameters in Yotta Pay API response.');
        }

        $responseStatus = $responseData['result'];

        if (!(boolval($responseStatus) && strtolower($responseStatus) != 'false'))
        {
            throw new \Exception('Request has been failed.');
        }
    }

    /**
     * Create data to verify loyalty programm status request
     *
     * @param string $merchantId
     * @param string $merchantKey
     * @return array
     */
    private function getDataToSendOld($merchantId, $merchantKey)
    {
        $body = [
            'type' => 'leEnabled',
            'merchantIdentifier' => $merchantId,
            'randomPayload' => uniqid(),
            'signature' => ''
        ];

        $dataToSign = (
            $body['merchantIdentifier']
            . $body['randomPayload']
            . $body['randomPayload']
            . $body['type']
            . $merchantKey
        );

        $sign = hash('sha256', $dataToSign);

        $body['signature'] = $sign;

        return $body;
    }
}
