<?php

namespace YottaPayIntegration\Payments\Observer;

/**
 * Yotta Pay SendLoyaltyEmail observer
 */
class SendLoyaltyEmail implements \Magento\Framework\Event\ObserverInterface
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \Magento\Framework\Mail\Template\TransportBuilder
     */
    protected $_transportBuilder;

    /**
     * @var \YottaPayIntegration\Payments\Model\Common
     */
    protected $_common;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder
     * @param \YottaPayIntegration\Payments\Model\Common $common
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder,
        \YottaPayIntegration\Payments\Model\Common $common
    ) {
        $this->_logger = $logger;
        $this->_transportBuilder = $transportBuilder;
        $this->_common = $common;
    }
    
    /**
     * SendLoyaltyEmail
     *
     * @param \Magento\Framework\Event\Observer $observer
     * @return void
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {   
        try
        {
            $this->_logger->debug('YottaPay | SendLoyaltyEmail execute | Start');

            $order = $observer->getEvent()->getOrder();
            $this->sendLoyaltyInfoEmail($order);
        }
        catch (\Exception $e)
        {
            $this->_logger->debug('YottaPay | SendLoyaltyEmail execute | ' . $e->getMessage());
        }
        finally
        {
            $this->_logger->debug('YottaPay | SendLoyaltyEmail execute | End');
        }
    }

    /**
     * Send new order loyalty info email
     *
     * @param \Magento\Sales\Model\Order $order
     * @return boolean
     */
    private function sendLoyaltyInfoEmail($order)
    {
        if (!($this->_common->getParamActive()))
        {
            throw new \Exception('Payment method is disabled in admin panel.');
        }            

        //Verify order status
        $orderStatus = $order->getState();
        if ($orderStatus != 'processing' && $orderStatus != 'complete')
        {
            throw new \Exception('Order status is not processing or complete.');
        }

        //Verify loyalty bonus
        if ($this->_common->getParamLoyaltyBonus() == 0) {
            throw new \Exception('Loyalty Bonus is disabled in admin panel.');
        }

        //Compare order created date and loyalty_start_period option
        if (!($this->_common->validateOrderDateMatchLoyalty($order)))
        {
            throw new \Exception('Too old order.');
        }

        if ($this->_common->usedOldApiKeys())
        {
            //Perform request to Yotta Pay API
            $responseData = $this->_common->performPostRequest(
                \YottaPayIntegration\Payments\Model\Common::$createDeferredEndpointOld,
                json_encode($this->getDataToSendOld($order))
            );

            //Process response data from Yotta Pay API
            $templateData = $this->processResponseDataOld($order, $responseData);
        }
        else
        {
            //Perform request to Yotta Pay API
            $responseData = $this->_common->performPostRequest(
                \YottaPayIntegration\Payments\Model\Common::$createDeferredEndpoint,
                json_encode($this->getDataToSend($order))
            );

            //Process response data from Yotta Pay API
            $templateData = $this->processResponseData($order, $responseData);
        }
            
        //Send from store
        $this->sendEmailLoyalty($order, $templateData);
    }

    /**
     * Create data to create deferred request
     *
     * @param \Magento\Sales\Model\Order $order
     * @return array
     */
    private function getDataToSendOld($order)
    {
        $baseUrl = $this->_common->getBaseUrl();

        $merchantId = $this->_common->getParamMerchantId();
        $merchantKey = $this->_common->getParamMerchantKey();

        //Get clientUserId
        $clientUserId = $this->_common->getClientUserId($order);

        //Get loyalty amount
        $loyaltyAmount = $this->_common->getLoyaltyAmount($order);

        $body = [
            'type' => 'accrual',
            'merchantIdentifier' => $merchantId,
            'clientUserId' => $clientUserId,
            'amount' => $loyaltyAmount,
            'comment' => $order->getId(),
            'notificationUrl' => (
                $baseUrl
                . \YottaPayIntegration\Payments\Model\Common::$apiLoyaltyResultOld
            ),
            'signature' => ''
        ];

        $dataToSign = (
            $body['merchantIdentifier']
            . $body['clientUserId']
            . $body['amount']
            . $body['comment']
            . $body['notificationUrl']
            . $merchantKey
        );

        $sign = hash('sha256', $dataToSign);

        $body['signature'] = $sign;

        return $body;
    }

    /**
     * Create data to create deferred request
     *
     * @param \Magento\Sales\Model\Order $order
     * @return array
     */
    private function getDataToSend($order)
    {
        $userToken = $this->_common->getUserToken();
        $clientUserId = $this->_common->getClientUserId($order);
        $loyaltyAmount = $this->_common->getLoyaltyAmount($order);

        $body = [
            'userAccessToken' => $userToken,
            'customerId' => $clientUserId,
            'amount' => $loyaltyAmount,
            'comment' => $order->getId(),
        ];

        $this->_logger->debug('YottaPay | CreateDeferred getDataToSend | body: ' . print_r($body, TRUE));

        return $body;
    }

    /**
     * Process a response data
     *
     * @param \Magento\Sales\Model\Order $order
     * @param array $responseData
     * @return array
     */
    private function processResponseDataOld($order, $responseData)
    {
        if (!array_key_exists('link', $responseData))
        {
            throw new \Exception('Missing required parameters in Yotta Pay API response.');
        }

        $templateVars = [
            'store' => $order->getStore(),
            'order_increment_id' => $order->getIncrementId(),
            'customer_name' => $order->getCustomerName(),
            'loyalty_bonus' => $this->_common->getParamLoyaltyBonus(),
            'create_deferred_link' => $responseData['link'],
        ];

        $templateOptions = [
            'area' => \Magento\Framework\App\Area::AREA_FRONTEND,
            'store' => $this->_common->getStoreId()
        ];

        $templateData = array(
            'vars' => $templateVars,
            'options' => $templateOptions
        );

        return $templateData;
    }

    /**
     * Process a response data
     *
     * @param \Magento\Sales\Model\Order $order
     * @param array $responseData
     * @return array
     */
    private function processResponseData($order, $responseData)
    {
        if (!array_key_exists('result', $responseData))
        {
            throw new \Exception('Missing required parameters in Yotta Pay API response.');
        }

        $templateVars = [
            'store' => $order->getStore(),
            'order_increment_id' => $order->getIncrementId(),
            'customer_name' => $order->getCustomerName(),
            'loyalty_bonus' => $this->_common->getParamLoyaltyBonus(),
            'create_deferred_link' => $responseData['result'],
        ];

        $templateOptions = [
            'area' => \Magento\Framework\App\Area::AREA_FRONTEND,
            'store' => $this->_common->getStoreId()
        ];

        $templateData = array(
            'vars' => $templateVars,
            'options' => $templateOptions
        );

        return $templateData;
    }

    /**
     * Send loyaty email from store
     *
     * @param \Magento\Sales\Model\Order $order
     * @param array $templateData
     */
    private function sendEmailLoyalty($order, $templateData)
    {
        $emailIdentity = $this->_common->getParamEmailIdentity();

        $this->_transportBuilder->setTemplateIdentifier('yottapay_email_loyalty');
        $this->_transportBuilder->setTemplateOptions($templateData['options']);
        $this->_transportBuilder->setTemplateVars($templateData['vars']);
        $this->_transportBuilder->setFromByScope($emailIdentity, $this->_common->getStoreId());
        $this->_transportBuilder->addTo($order->getCustomerEmail(), $order->getCustomerName());

        $transport = $this->_transportBuilder->getTransport();
        $transport->sendMessage();
    }
}
